/*********************************************************
This is a library for the MPR121 12-channel Capacitive touch sensor

Designed specifically to work with the MPR121 Breakout in the Adafruit shop 
  ----> https://www.adafruit.com/products/

These sensors use I2C communicate, at least 2 pins are required 
to interface

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada for Adafruit Industries.  
BSD license, all text above must be included in any redistribution
**********************************************************/

#include <Wire.h>
#include "Adafruit_MPR121.h"
#include <ESP8266WiFi.h>

#ifndef _BV
#define _BV(bit) (1 << (bit)) 
#endif

// WiFi parameters
const char* ssid = "INSERT CODE HERE"; //replace with your ssid
const char* password = "INSERT CODE HERE";//replace with your pw

char MakerIFTTT_Key[] = "INSERT CODE HERE";
char MakerIFTTT_Event[] = "ITO_touch"; //

WiFiClient client;

// You can have up to 4 on one i2c bus but one is enough for testing!
Adafruit_MPR121 cap = Adafruit_MPR121();

// Keeps track of the last pins touched
// so we know when buttons are 'released'
static uint16_t lasttouched = 0;
static uint16_t currtouched = 0;

void setupWiFi() {
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.print(ssid);
  Serial.println("...");

  WiFi.begin(ssid, password);
  int retries = 0;
  while ((WiFi.status() != WL_CONNECTED) && (retries < 15)) {
    retries++;
    delay(500);
    Serial.print(".");
  }
  if (retries > 14) {
    Serial.println(F("WiFi conenction FAILED"));
  }
  if (WiFi.status() == WL_CONNECTED) {
    Serial.println(F("WiFi connected!"));
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
  }
  Serial.println(F("Setup ready"));
}

void setup() {
  Serial.begin(115200);
  setupWiFi();
  while (!Serial) { // needed to keep leonardo/micro from starting too fast!
    delay(10);
  }
  
  Serial.println("Adafruit MPR121 Capacitive Touch sensor test"); 
  
  // Default address is 0x5A, if tied to 3.3V its 0x5B
  // If tied to SDA its 0x5C and if SCL then 0x5D
  if (!cap.begin(0x5A)) {
    Serial.println("MPR121 not found, check wiring?");
    while (1);
  }
  Serial.println("MPR121 found!");
}

void loop() {
  // Get the currently touched pads
  currtouched = cap.touched();


  int nITO_data = 1;
  bool ITO_status = get_ITO_data(nITO_data);

  if(!ITO_status){
    updateEvent();
  }


  // reset our state
  lasttouched = currtouched;
  
  delay(1000);
}


char *append_str(char *here, char *s) {
  while (*here++ = *s++)
    ;
  return here - 1;
}
char *append_ul(char *here, unsigned long u) {
  char buf[20];       // we "just know" this is big enough

  return append_str(here, ultoa(u, buf, 10));
}

// Send a POST to trigger the IFTTT/Maker event
void updateEvent() {
  // connect to the Maker event server
  client.connect("maker.ifttt.com", 80);
 
  // construct the POST request
  char post_rqst[256];    // hand-calculated to be big enough

  char *p = post_rqst;
  p = append_str(p, "POST /trigger/");
  p = append_str(p, MakerIFTTT_Event);
  p = append_str(p, "/with/key/");
  p = append_str(p, MakerIFTTT_Key);
  p = append_str(p, " HTTP/1.1\r\n");
  p = append_str(p, "Host: maker.ifttt.com\r\n");
  p = append_str(p, "Content-Type: application/json\r\n");
  p = append_str(p, "Content-Length: ");



  char *content_length_here = p; // we need to remember where the content length will go, which is:

  p = append_str(p, "NN\r\n"); // it's always two digits, so reserve space for them (the NN)
  p = append_str(p, "\r\n"); // end of headers
  char *json_start = p; // construct the JSON; remember where we started so we will know len

      // As described - this example reports a pin, uptime, and "hello world"
    Serial.println("Sending values to IFTTT");
//    p = append_str(p, "{\"value1\":\"");
//    p = append_str(p, lat_lng);
//    p = append_str(p, "\"}");
      // go back and fill in the JSON length
      // we just know this is at most 2 digits (and need to fill in both)
      int i = strlen(json_start);
      content_length_here[0] = '0' + (i/10);
      content_length_here[1] = '0' + (i%10);
  
      // finally we are ready to send the POST to the server!
      client.print(post_rqst);
      String line = client.readStringUntil('\n');
      Serial.println(line);
      client.stop();
      delay(10000);
}

int get_ITO_data(int connection){
     int i = connection;
    if ((currtouched & _BV(i)) && !(lasttouched & _BV(i)) ) {
      Serial.print(i); Serial.println(" touched") ;     
      return 1;
    }
    // if it *was* touched and now *isnt*, alert!
    if (!(currtouched & _BV(i)) && (lasttouched & _BV(i)) ) {
      Serial.print(i); Serial.println(" released");
      return 0;
    }
  
}
